==> iRRAM_2003_03.tar.gz: iRRAM_2003_03/include/GMP_interface.h <==
/*

GMP_interface.h -- interface to the GMP float functions for the iRRAM library
 
Copyright (C) 2002 Norbert Mueller
 
This file is part of the iRRAM Library.
 
The iRRAM Library is free software; you can redistribute it and/or modify
it under the terms of the GNU Library General Public License as published by
the Free Software Foundation; either version 2 of the License, or (at your
option) any later version.
 
The iRRAM Library is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Library General Public
License for more details.
 
You should have received a copy of the GNU Library General Public License
along with the iRRAM Library; see the file COPYING.LIB.  If not, write to
the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
MA 02111-1307, USA. 
*/

#include "gmp.h"

#include "GMP_ext.h"
#include "GMP_int_ext.h"
#include "GMP_rat_ext.h"

#define MP_min    GMP_min
#define MP_max    GMP_max

#define MP_int_getsize(z) 		int_gmp_getsize(z)
#define MP_int_binsize(z)                  mpz_sizeinbase(z,2)

/****** Type definitions ******/

#define MP_type			ext_gmp_type
#define MP_int_type 		int_gmp_type
#define MP_rat_type		rat_gmp_type

/****** Initialization functions ******/

#define MP_initialize		ext_gmp_initialize()

#define MP_init(z)		z=ext_gmp_init()
#define MP_int_init(z)		z=int_gmp_init()
#define MP_rat_init(z)		z=rat_gmp_init()

#define MP_clear(z)		ext_gmp_free(z) 
#define MP_int_clear(z)		int_gmp_free(z)
#define MP_rat_clear(z)		rat_gmp_free(z)


/****** typechanging functions ******/
#define MP_mp_to_double(z)          mpf_get_d(z)
#define MP_double_to_mp(d,z)        mpf_set_d(z,d)

#define MP_int_to_mp(i,z) mpf_set_si(z,i)
#define MP_int_to_INTEGER(i,z) 	int_gmp_int2int(i,z)

#define MP_INTEGER_to_mp(i,r) ext_gmp_set_z(r,i)
#define MP_INTEGER_to_long(z) 	int_gmp_int2long(z)

#define MP_mp_to_INTEGER(r,i) mpz_set_f(i,r)

#define MP_double_to_INTEGER(d,z) 	int_gmp_double2int(d,z)
#define MP_string_to_INTEGER(s,z,b) int_gmp_string2int(s,z,b)

#define MP_int_to_RATIONAL(i,z) mpq_set_si(z,i,1)
#define MP_intint_to_RATIONAL(i,j,z) mpq_set_si(z,i,j);mpq_canonicalize(z)
#define MP_double_to_RATIONAL(d,z) mpq_set_d(z,d)
#define MP_string_to_RATIONAL(s,z) rat_gmp_string_2_rat(z,s)
#define MP_INTEGER_to_RATIONAL(i,r) mpq_set_z(i,r)
#define MP_INTINTEGER_to_RATIONAL(i,j,r) mpq_set_num(r,i);mpq_set_den(r,j);mpq_canonicalize(r)


// duplicate value z1 to z2, with/without initialization of z2 
#define MP_duplicate_w_init(z1,z2) 		ext_gmp_duplicate_w_init(z1,&(z2))
#define MP_duplicate_wo_init(z1,z2)		ext_gmp_duplicate_wo_init(z1,z2) 

#define MP_int_duplicate_w_init(z1,z2)  	int_gmp_duplicate_w_init(z1,&(z2))
#define MP_int_duplicate_wo_init(z1,z2) 	int_gmp_duplicate_wo_init(z1,z2) 

#define MP_rat_duplicate_w_init(z1,z2)  	rat_gmp_duplicate_w_init(z1,&(z2))
#define MP_rat_duplicate_wo_init(z1,z2) 	rat_gmp_duplicate_wo_init(z1,z2) 

// copy z1 to z2, but precision p is sufficient
#define MP_copy(z1,z2,p) ext_gmp_duplicate_wo_init(z1,z2)


/****** simple arithmetic functions ******/

#define MP_add(z1,z2,z,p) ext_gmp_add(z1,z2,z,p)
#define MP_sub(z1,z2,z,p) ext_gmp_sub(z1,z2,z,p)
#define MP_mul(z1,z2,z,p) ext_gmp_mul(z1,z2,z,p) 
#define MP_div(z1,z2,z,p) ext_gmp_div(z1,z2,z,p) 
#define MP_abs(z1,z) ext_gmp_abs(z1,z)
#define MP_addi(z1,z2,z,p) ext_gmp_add_i(z1,z2,z,p)
#define MP_subi(z1,z2,z,p) ext_gmp_sub_i(z1,z2,z,p)
#define MP_isub(z1,z2,z,p) ext_gmp_i_sub(z1,z2,z,p)
#define MP_muli(z1,z2,z,p) ext_gmp_mul_i(z1,z2,z,p) 
#define MP_divi(z1,z2,z,p) ext_gmp_div_i(z1,z2,z,p)
#define MP_idiv(z1,z2,z,p) ext_gmp_i_div(z1,z2,z,p); 

#define MP_int_add(z1,z2,z) 		int_gmp_add(z1,z2,z)
#define MP_int_sub(z1,z2,z) 		int_gmp_sub(z1,z2,z)
#define MP_int_mul(z1,z2,z) 		int_gmp_mul(z1,z2,z)
#define MP_int_div(z1,z2,z) 		int_gmp_div(z1,z2,z)
#define MP_int_add_ui(z1,z2,z)	int_gmp_add_ui(z1,z2,z)
#define MP_int_sub_ui(z1,z2,z)	int_gmp_sub_ui(z1,z2,z)
#define MP_int_mul_si(z1,z2,z)	int_gmp_mul_si(z1,z2,z)
#define MP_int_div_ui(z1,z2,z)	int_gmp_div_ui(z1,z2,z)
#define MP_int_abs(z1,z) 		int_gmp_abs(z1,z);
#define MP_int_neg(z1,z);		int_gmp_neg(z1,z);

#define MP_rat_add(z1,z2,z) rat_gmp_add(z1,z2,z)
#define MP_rat_sub(z1,z2,z) rat_gmp_sub(z1,z2,z)
#define MP_rat_mul(z1,z2,z) rat_gmp_mul(z1,z2,z)
#define MP_rat_div(z1,z2,z) rat_gmp_div(z1,z2,z) 
#define MP_rat_add_ui(z1,z2,z) rat_gmp_add_ui(z1,z2,z)
#define MP_rat_sub_ui(z1,z2,z) rat_gmp_sub_ui(z1,z2,z)
#define MP_rat_mul_si(z1,z2,z) rat_gmp_mul_si(z1,z2,z)
#define MP_rat_div_si(z1,z2,z) rat_gmp_div_si(z1,z2,z)
#define MP_rat_si_div(z1,z2,z) rat_gmp_si_div(z1,z2,z)
#define MP_rat_abs(z1,z) rat_gmp_abs(z1,z);

#define MP_int_sqrt(z1,z) 		int_gmp_sqrt(z1,z);
#define MP_int_root(z1,z2,z) 		int_gmp_root(z1,z2,z);
#define MP_int_power_i(z1,z2,z) 	int_gmp_power_i(z1,z2,z);
#define MP_int_power_ii(z1,z2,z) 	int_gmp_power_ii(z1,z2,z);
#define MP_int_fac(z1,z) 		int_gmp_fac(z1,z);
#define MP_int_modulo(z1,z2,z) 	int_gmp_modulo(z1,z2,z);
#define MP_int_log(z) 			int_gmp_log(z);

#define MP_rat_power(z1,z2,z) rat_gmp_power(z1,z2,z);
#define MP_rat_power2(z1,z2,z) rat_gmp_power_ii(z1,z2,z);
#define MP_rat_size(z) rat_gmp_size(z);
#define MP_rat_log(z) rat_gmp_log(z);
#define MP_rat_getsize(z) rat_gmp_getsize(z)
#define MP_rat_binsize(z) rat_gmp_binsize(z)
#define MP_rat_get_numerator(z1,z) mpq_get_num(z,z1) 
#define MP_rat_get_denominator(z1,z) mpq_get_den(z,z1)
#define MP_rat_canon(z) mpq_canonicalize(z) //rat_gmp_canon(z)

// MP_compare: values -1 (lessthan), 0 (equal), or 1 (greater than)
#define MP_compare(z1,z2) mpf_cmp(z1,z2)
#define MP_int_compare(z1,z2) 	int_gmp_cmp(z1,z2)
#define MP_rat_compare(z1,z2) mpq_cmp(z1,z2)

// MP_sign: values -1 (negative), 0 (zero), or 1 (positive)
#define MP_sign(z) mpf_sgn(z)
#define MP_int_sign(z) 			int_gmp_sgn(z)
#define MP_rat_sign(z) mpq_sgn(z)

// long MP_size : smallest(?) value k such that 2**k > z
#define MP_size(z) ext_gmp_size(z)
#define MP_getsize(z,s) ext_gmp_getsize(z,(ext_gmp_sizetype*)&s)
#define MP_int_size(z) 				int_gmp_size(z)
#define MP_int_getsize(z) 			int_gmp_getsize(z)

// truncate z1 yielding z2 
#define MP_truncate(z1,z) ext_gmp_truncate(z1,z)

//Conversion to strings
#define MP_swrite(z,w)		ext_gmp_swritee(z,w)

// In/Out
#define MP_writee(z,w) ext_gmp_writee(z,w)
#define MP_writef(z,w,d) mpf_out_str(stdout,10,w,z)
#define MP_write(z,c) ext_gmp_write(z,c)

#define MP_int_writee(z,w) 		int_gmp_writee(z,w)
#define MP_int_swritee(z,w)		int_gmp_swritee(z,w)
#define MP_int_writef(z,w) 		int_gmp_outstr(z,w)
#define MP_int_write(z,c) 		int_gmp_write(z,c)
#define MP_int_printf(z) 		int_gmp_printf(z)
#define MP_int_sprintf(z)		int_gmp_sprintf(z)

#define MP_rat_writee(z,w) rat_gmp_writee(z,w)
#define MP_rat_writef(z,w) rat_gmp_outstr(z,w)
#define MP_rat_swritee(z,w) rat_gmp_swritee(z,w)
#define MP_rat_sprintf(z) rat_gmp_sprintf(z)

// Statistics

#define MP_var_count ext_gmp_var_count
//#define MP_space_count 
//#define MP_max_space_count

// special functions
#define MP_has_sqrt 1
#define MP_sqrt(z1,z,p) ext_gmp_sqrt(z1,z,p)

#define MP_has_shift 1
#define MP_shift(z1,z,n) ext_gmp_shift(z1,z,n)

#define MP_has_int_shift 1
#define MP_int_shift(z1,z,n) 		int_gmp_shift(z1,z,n)

#define MP_has_rat_shift 1
#define MP_rat_shift(z,z1,n) rat_gmp_shift(z1,z,n)

==> iRRAM_2003_05.tar.gz: iRRAM_2003_05/include/GMP_interface.h <==
/*

GMP_interface.h -- interface to the GMP float functions for the iRRAM library
 
Copyright (C) 2002 Norbert Mueller
 
This file is part of the iRRAM Library.
 
The iRRAM Library is free software; you can redistribute it and/or modify
it under the terms of the GNU Library General Public License as published by
the Free Software Foundation; either version 2 of the License, or (at your
option) any later version.
 
The iRRAM Library is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Library General Public
License for more details.
 
You should have received a copy of the GNU Library General Public License
along with the iRRAM Library; see the file COPYING.LIB.  If not, write to
the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
MA 02111-1307, USA. 
*/

#include "gmp.h"

#include "GMP_ext.h"
#include "GMP_intrat.h"

#define MP_min    GMP_min
#define MP_max    GMP_max

#define MP_int_getsize(z) 		int_gmp_getsize(z)
#define MP_int_binsize(z)                  mpz_sizeinbase(z,2)

/****** Type definitions ******/

#define MP_type			ext_gmp_type
#define MP_int_type 		int_gmp_type
#define MP_rat_type		rat_gmp_type

/****** Initialization functions ******/

#define MP_initialize		ext_gmp_initialize()

#define MP_init(z)		z=ext_gmp_init()
#define MP_int_init(z)		z=int_gmp_init()
#define MP_rat_init(z)		z=rat_gmp_init()

#define MP_clear(z)		ext_gmp_free(z) 
#define MP_int_clear(z)		int_gmp_free(z)
#define MP_rat_clear(z)		rat_gmp_free(z)


/****** typechanging functions ******/
#define MP_mp_to_double(z)          mpf_get_d(z)
#define MP_double_to_mp(d,z)        mpf_set_d(z,d)

#define MP_int_to_mp(i,z) mpf_set_si(z,i)
#define MP_int_to_INTEGER(i,z) 	int_gmp_int2int(i,z)

#define MP_INTEGER_to_mp(i,r) ext_gmp_set_z(r,i)
#define MP_INTEGER_to_long(z) 	int_gmp_int2long(z)

#define MP_mp_to_INTEGER(r,i) mpz_set_f(i,r)

#define MP_double_to_INTEGER(d,z) 	int_gmp_double2int(d,z)
#define MP_string_to_INTEGER(s,z,b) int_gmp_string2int(s,z,b)

#define MP_int_to_RATIONAL(i,z) mpq_set_si(z,i,1)
#define MP_intint_to_RATIONAL(i,j,z) mpq_set_si(z,i,j);mpq_canonicalize(z)
#define MP_double_to_RATIONAL(d,z) mpq_set_d(z,d)
#define MP_string_to_RATIONAL(s,z) rat_gmp_string_2_rat(z,s)
#define MP_INTEGER_to_RATIONAL(i,r) mpq_set_z(r,i)
#define MP_INTINTEGER_to_RATIONAL(i,j,r) mpq_set_num(r,i);mpq_set_den(r,j);mpq_canonicalize(r)


// duplicate value z1 to z2, with/without initialization of z2 
#define MP_duplicate_w_init(z1,z2) 		ext_gmp_duplicate_w_init(z1,&(z2))
#define MP_duplicate_wo_init(z1,z2)		ext_gmp_duplicate_wo_init(z1,z2) 

#define MP_int_duplicate_w_init(z1,z2)  	int_gmp_duplicate_w_init(z1,&(z2))
#define MP_int_duplicate_wo_init(z1,z2) 	int_gmp_duplicate_wo_init(z1,z2) 

#define MP_rat_duplicate_w_init(z1,z2)  	rat_gmp_duplicate_w_init(z1,&(z2))
#define MP_rat_duplicate_wo_init(z1,z2) 	rat_gmp_duplicate_wo_init(z1,z2) 

// copy z1 to z2, but precision p is sufficient
#define MP_copy(z1,z2,p) ext_gmp_duplicate_wo_init(z1,z2)


/****** simple arithmetic functions ******/

#define MP_add(z1,z2,z,p) ext_gmp_add(z1,z2,z,p)
#define MP_sub(z1,z2,z,p) ext_gmp_sub(z1,z2,z,p)
#define MP_mul(z1,z2,z,p) ext_gmp_mul(z1,z2,z,p) 
#define MP_div(z1,z2,z,p) ext_gmp_div(z1,z2,z,p) 
#define MP_abs(z1,z) ext_gmp_abs(z1,z)
#define MP_addi(z1,z2,z,p) ext_gmp_add_i(z1,z2,z,p)
#define MP_subi(z1,z2,z,p) ext_gmp_sub_i(z1,z2,z,p)
#define MP_isub(z1,z2,z,p) ext_gmp_i_sub(z1,z2,z,p)
#define MP_muli(z1,z2,z,p) ext_gmp_mul_i(z1,z2,z,p) 
#define MP_divi(z1,z2,z,p) ext_gmp_div_i(z1,z2,z,p)
#define MP_idiv(z1,z2,z,p) ext_gmp_i_div(z1,z2,z,p); 

#define MP_int_add(z1,z2,z) 		int_gmp_add(z1,z2,z)
#define MP_int_sub(z1,z2,z) 		int_gmp_sub(z1,z2,z)
#define MP_int_mul(z1,z2,z) 		int_gmp_mul(z1,z2,z)
#define MP_int_div(z1,z2,z) 		int_gmp_div(z1,z2,z)
#define MP_int_add_ui(z1,z2,z)	int_gmp_add_ui(z1,z2,z)
#define MP_int_sub_ui(z1,z2,z)	int_gmp_sub_ui(z1,z2,z)
#define MP_int_mul_si(z1,z2,z)	int_gmp_mul_si(z1,z2,z)
#define MP_int_div_ui(z1,z2,z)	int_gmp_div_ui(z1,z2,z)
#define MP_int_abs(z1,z) 		int_gmp_abs(z1,z);
#define MP_int_neg(z1,z);		int_gmp_neg(z1,z);

#define MP_rat_add(z1,z2,z) rat_gmp_add(z1,z2,z)
#define MP_rat_sub(z1,z2,z) rat_gmp_sub(z1,z2,z)
#define MP_rat_mul(z1,z2,z) rat_gmp_mul(z1,z2,z)
#define MP_rat_div(z1,z2,z) rat_gmp_div(z1,z2,z) 
#define MP_rat_add_ui(z1,z2,z) rat_gmp_add_ui(z1,z2,z)
#define MP_rat_sub_ui(z1,z2,z) rat_gmp_sub_ui(z1,z2,z)
#define MP_rat_mul_si(z1,z2,z) rat_gmp_mul_si(z1,z2,z)
#define MP_rat_div_si(z1,z2,z) rat_gmp_div_si(z1,z2,z)
#define MP_rat_si_div(z1,z2,z) rat_gmp_si_div(z1,z2,z)
#define MP_rat_abs(z1,z) rat_gmp_abs(z1,z);

#define MP_int_sqrt(z1,z) 		int_gmp_sqrt(z1,z);
#define MP_int_root(z1,z2,z) 		int_gmp_root(z1,z2,z);
#define MP_int_power_i(z1,z2,z) 	int_gmp_power_i(z1,z2,z);
#define MP_int_power_ii(z1,z2,z) 	int_gmp_power_ii(z1,z2,z);
#define MP_int_fac(z1,z) 		int_gmp_fac(z1,z);
#define MP_int_modulo(z1,z2,z) 	int_gmp_modulo(z1,z2,z);
#define MP_int_log(z) 			int_gmp_log(z);

#define MP_rat_power(z1,z2,z) rat_gmp_power(z1,z2,z);
#define MP_rat_power2(z1,z2,z) rat_gmp_power_ii(z1,z2,z);
#define MP_rat_size(z) rat_gmp_size(z);
#define MP_rat_log(z) rat_gmp_log(z);
#define MP_rat_getsize(z) rat_gmp_getsize(z)
#define MP_rat_binsize(z) rat_gmp_binsize(z)
#define MP_rat_get_numerator(z1,z) mpq_get_num(z,z1) 
#define MP_rat_get_denominator(z1,z) mpq_get_den(z,z1)
#define MP_rat_canon(z) mpq_canonicalize(z) //rat_gmp_canon(z)

// MP_compare: values -1 (lessthan), 0 (equal), or 1 (greater than)
#define MP_compare(z1,z2) mpf_cmp(z1,z2)
#define MP_int_compare(z1,z2) 	int_gmp_cmp(z1,z2)
#define MP_rat_compare(z1,z2) mpq_cmp(z1,z2)

// MP_sign: values -1 (negative), 0 (zero), or 1 (positive)
#define MP_sign(z) mpf_sgn(z)
#define MP_int_sign(z) 			int_gmp_sgn(z)
#define MP_rat_sign(z) mpq_sgn(z)

// long MP_size : smallest(?) value k such that 2**k > z
#define MP_size(z) ext_gmp_size(z)
#define MP_getsize(z,s) ext_gmp_getsize(z,(ext_gmp_sizetype*)&s)
#define MP_int_size(z) 				int_gmp_size(z)
#define MP_int_getsize(z) 			int_gmp_getsize(z)

// truncate z1 yielding z2 
#define MP_truncate(z1,z) ext_gmp_truncate(z1,z)

//Conversion to strings
#define MP_swrite(z,w)		ext_gmp_swritee(z,w)

// In/Out
#define MP_writee(z,w) ext_gmp_writee(z,w)
#define MP_writef(z,w,d) mpf_out_str(stdout,10,w,z)
#define MP_write(z,c) ext_gmp_write(z,c)

#define MP_int_writee(z,w) 		int_gmp_writee(z,w)
#define MP_int_swritee(z,w)		int_gmp_swritee(z,w)
#define MP_int_writef(z,w) 		int_gmp_outstr(z,w)
#define MP_int_write(z,c) 		int_gmp_write(z,c)
#define MP_int_printf(z) 		int_gmp_printf(z)
#define MP_int_sprintf(z)		int_gmp_sprintf(z)

#define MP_rat_writee(z,w) rat_gmp_writee(z,w)
#define MP_rat_writef(z,w) rat_gmp_outstr(z,w)
#define MP_rat_swritee(z,w) rat_gmp_swritee(z,w)
#define MP_rat_sprintf(z) rat_gmp_sprintf(z)

// Statistics, optional

#define MP_var_count ext_gmp_var_count
//#define MP_space_count 
//#define MP_max_space_count

// special functions, optional
#define MP_sqrt(z1,z,p) ext_gmp_sqrt(z1,z,p)
#define MP_shift(z1,z,n) ext_gmp_shift(z1,z,n)
#define MP_int_shift(z1,z,n) 		int_gmp_shift(z1,z,n)
#define MP_rat_shift(z,z1,n) rat_gmp_shift(z1,z,n)
